/*
 * DuneApp.h
 *
 * Copyright (C) 1999 Stephen F. White
 * 
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program (see the file "COPYING" for details); if 
 * not, write to the Free Software Foundation, Inc., 675 Mass Ave, 
 * Cambridge, MA 02139, USA.
 */

#ifndef _DUNE_APP_H
#define _DUNE_APP_H

#ifndef _LIST_H
#include "List.h"
#endif
#ifndef _ARRAY_H
#include "Array.h"
#endif
#ifndef _DUNE_STRING_H
#include "String.h"
#endif

class Scene;
class MainWindow;

typedef enum {
    HM_NONE,
    HM_SELECTED,
    HM_ALL,
} HandleMode;

typedef struct STable *STABLE;
typedef struct SBrowser *SBROWSER;

class DuneApp {
public:
			DuneApp();

    void		OnFileNew();
    void		OnFileClose(MainWindow *window);
    void		OnFileExit();

    bool		OpenFile(const char *path);

    HandleMode		GetHandleMode() const { return _handleMode; }
    bool		GetShowAllFields() const { return _showAllFields; }
    int			GetRotationOrder() const { return _rotationOrder; }
    const char	       *GetRotationTitle() const { return _rotationTitle; }

    void    		SetHandleMode(HandleMode handleMode);
    void		SetShowAllFields(bool s);
    void		SetRotationOrder(int rotationOrder);
    void		SetRotationTitle(const char *s)
						{ _rotationTitle = s; }

    bool		GetBoolPreference(const char *key, bool defaultValue);
    int			GetIntPreference(const char *key, int defaultValue);
    const char	       *GetPreference(const char *key,
    				      const char *defaultValue);

    void		SetBoolPreference(const char *key, bool value);
    void		SetIntPreference(const char *key, int value);
    void		SetPreference(const char *key, const char *value);

    int			GetNumRecentFiles() const;
    const String       &GetRecentFile(int index) const;
    void		AddToRecentFiles(const String &filename);

    SBROWSER		GetBrowser() const { return _browser; }

protected:
    void		UpdateAllWindows();
    void		SaveRecentFileList();
    void		Exit();

protected:
    List<MainWindow *>	_windows;
    bool		_showAllFields;
    int			_rotationOrder;
    const char	       *_rotationTitle;
    HandleMode		_handleMode;
    SBROWSER		_browser;
    STABLE		_prefs;
    Array<String>	_recentFiles;
};

extern DuneApp *TheApp;

#endif // _DUNE_APP_H
