/*
 * MFFloat.cpp
 *
 * Copyright (C) 1999 Stephen F. White
 * 
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program (see the file "COPYING" for details); if 
 * not, write to the Free Software Foundation, Inc., 675 Mass Ave, 
 * Cambridge, MA 02139, USA.
 */

#include <stdio.h> 
#include "stdafx.h"

#include "MFFloat.h"

MFFloat::MFFloat(float *values, int len, int /* stride */)
{
    _value = values;
    _len = len;
}

MFFloat::MFFloat(int /* stride */)
{
    _value = NULL;
    _len = 0;
}

MFFloat::MFFloat(const MFFloat &value)
{
    _value = new float[value._len];
    _len = value._len;
    memcpy(_value, value._value, _len * sizeof(float));
}

MFFloat::~MFFloat()
{
    delete [] _value;
}

bool
MFFloat::equals(const FieldValue *value) const
{
    return value->getType() == MFFLOAT && equals((const MFFloat *) value);
}

bool
MFFloat::equals(const MFFloat *value) const
{
    if (_len == value->getSize()) {
	for (int i = 0; i < _len; i++)
	    if (_value[i] != value->getValue(i))
		return false;
	return true;
    }
    return false;
}

void MFFloat::write(FILE *f, int indent) const
{
    int		stride = getStride();

    fprintf(f, "[\n");
    for (int i = 0; i < _len; i += stride) {
	indentf(f, indent+2);
	for (int j = 0; j < stride; j++) {
	    fprintf(f, "%g ", _value[i+j]);
	}
	fprintf(f, "\n");
    }
    indentf(f, indent);
    fprintf(f, "]\n");
}

void MFFloat::clamp(const FieldValue *min, const FieldValue *max)
{
    if (min) {
	float fmin = ((SFFloat *) min)->getValue();
	for (int i = 0; i < _len; i ++) {
	    if (_value[i] < fmin) _value[i] = fmin;
	}
    }
    if (max) {
	float fmax = ((SFFloat *) max)->getValue();
	for (int i = 0; i < _len; i ++) {
	    if (_value[i] > fmax) _value[i] = fmax;
	}
    }
}

void MFFloat::setSFValue(int index, FieldValue *value)
{
    _value[index] = ((SFFloat *) value)->getValue();
}

