/*
 * MFNode.cpp
 *
 * Copyright (C) 1999 Stephen F. White
 * 
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program (see the file "COPYING" for details); if 
 * not, write to the Free Software Foundation, Inc., 675 Mass Ave, 
 * Cambridge, MA 02139, USA.
 */

#include <stdio.h>
#include "stdafx.h"

#include "MFNode.h"
#include "SFNode.h"
#include "Node.h"

MFNode::MFNode()
{
    _value = new NodeList();
}

MFNode::MFNode(const MFNode &other)
{
    int		size = other._value->size();

    _value = new NodeList();
    for (int i = 0; i < size; i++) {
	_value->set(i, other._value->get(i)->copy());
	_value->get(i)->ref();
    }
}

MFNode::MFNode(NodeList *value)
{
    _value = value;
    for (int i = 0; i < _value->size(); i++)
	_value->get(i)->ref();
}

MFNode::~MFNode()
{
    for (int i = 0; i < _value->size(); i++)
	_value->get(i)->unref();
    delete _value;
}

void
MFNode::write(FILE *f, int indent) const
{
    fprintf(f, "[\n");
    for (int i = 0; i < _value->size(); i++) {
	_value->get(i)->write(f, indent + 2);
    }
    indentf(f, indent);
    fprintf(f, "]\n");
}

bool
MFNode::equals(const FieldValue *value) const
{
    if (value->getType() == MFNODE) {
	NodeList	*v = ((MFNode *) value)->getValues();
	if (v->size() != _value->size()) return false;
	for (int i = 0; i < _value->size(); i++)
	    if (v->get(i) != _value->get(i)) return false;
	return true;
    }
    return false;
}

FieldValue *
MFNode::addNode(Node *node) const
{
    NodeList    *list = new NodeList();
    for (int i = 0; i < _value->size(); i++) {
	list->append(_value->get(i));
    }
    list->append(node);
    return new MFNode(list);
}

FieldValue *
MFNode::removeNode(Node *node) const
{
    NodeList    *list = new NodeList();

    for (int i = 0; i < _value->size(); i++) {
	if (_value->get(i) != node) list->append(_value->get(i));
    }
    return new MFNode(list);
}

FieldValue *
MFNode::getSFValue(int index) const
{
    return new SFNode(_value->get(index));
}

void
MFNode::setSFValue(int index, FieldValue *value)
{
    _value[index] = ((SFNode *) value)->getValue();
}

