/*
 * MFTime.cpp
 *
 * Copyright (C) 1999 Stephen F. White
 * 
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program (see the file "COPYING" for details); if 
 * not, write to the Free Software Foundation, Inc., 675 Mass Ave, 
 * Cambridge, MA 02139, USA.
 */

#include <stdio.h> 
#include "stdafx.h"

#include "MFTime.h"
#include "SFTime.h"

MFTime::MFTime()
{
    _len = 0;
    _value = NULL;
}

MFTime::MFTime(const MFTime &value)
{
    _len = value._len;
    _value = new double[value._len];
    memcpy(_value, value._value, _len * sizeof(double));
}

MFTime::MFTime(double *value, int len)
{
    _value = value;
    _len = len;
}

bool
MFTime::equals(const FieldValue *value) const
{
    return value->getType() == MFTIME && equals((const MFTime *) value);
}

bool
MFTime::equals(const MFTime *value) const
{
    if (_len == value->_len) {
	for (int i = 0; i < _len; i++)
	    if (_value[i] != value->_value[i])
		return false;
	return true;
    }
    return false;
}

void MFTime::write(FILE *f, int indent) const
{
    fprintf(f, "[\n");
    for (int i = 0; i < _len; i++) {
	indentf(f, indent+2);
	fprintf(f, "%g\n", _value[i]);
    }
    indentf(f, indent);
    fprintf(f, "]\n");
}

void MFTime::clamp(const FieldValue *min, const FieldValue *max)
{
    if (min) {
	double	dmin = ((SFTime *) min)->getValue();
	for (int i = 0; i < _len; i++) {
	    if (_value[i] < dmin) _value[i] = dmin;
	}
    }
    if (max) {
	double	dmax = ((SFTime *) max)->getValue();
	for (int i = 0; i < _len; i++) {
	    if (_value[i] > dmax) _value[i] = dmax;
	}
    }
}

void MFTime::setSFValue(int index, FieldValue *value)
{
    _value[index] = ((SFTime *) value)->getValue();
}

