/*
 * NodeCollision.cpp
 *
 * Copyright (C) 1999 Stephen F. White
 * 
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program (see the file "COPYING" for details); if 
 * not, write to the Free Software Foundation, Inc., 675 Mass Ave, 
 * Cambridge, MA 02139, USA.
 */

#include <stdio.h>
#include <GL/gl.h>
#include "stdafx.h"

#include "NodeCollision.h"
#include "Proto.h"
#include "FieldValue.h"
#include "MFNode.h"
#include "SFBool.h"
#include "SFVec3f.h"
#include "SFNode.h"
#include "SFFloat.h"

ProtoCollision::ProtoCollision(Scene *scene)
  : Proto(scene, "Collision")
{
    addEventIn(MFNODE, "addChildren");
    addEventIn(MFNODE, "removeChildren");
    addExposedField(MFNODE, "children", new MFNode(), NODE_CHILD);
    addExposedField(SFBOOL, "collide", new SFBool(true));
    addField(SFVEC3F, "bboxCenter", new SFVec3f(0.0f, 0.0f, 0.0f));
    addField(SFVEC3F, "bboxSize", new SFVec3f(-1.0f, -1.0f, -1.0f), new SFFloat(-1.0f));
    addField(SFNODE, "proxy", new SFNode(NULL), NODE_CHILD);
    addEventOut(SFTIME, "collideTime");
}

NodeCollision::NodeCollision(Scene *scene, Proto *def)
  : Node(scene, def)
{
}

void
NodeCollision::draw()
{
    int		i;
    NodeList   *children = ((MFNode *) getField(0))->getValues();
    int		n = children->size();

    for (i = 0; i < n; i++)
	children->get(i)->bind();

    glPushName(0);			// field
    glPushName(0);			// index
    for (i = 0; i < n; i++) {
	glLoadName(i);
	children->get(i)->draw();
    }
    glPopName();
    glPopName();

    for (i = 0; i < n; i++)
	children->get(i)->unbind();
}

