/*
 * NodeFog.cpp
 *
 * Copyright (C) 1999 Stephen F. White
 * 
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program (see the file "COPYING" for details); if 
 * not, write to the Free Software Foundation, Inc., 675 Mass Ave, 
 * Cambridge, MA 02139, USA.
 */

#include <stdio.h>
#include <GL/gl.h>
#include "stdafx.h"

#include "NodeFog.h"
#include "Proto.h"
#include "Field.h"
#include "FieldValue.h"
#include "SFColor.h"
#include "SFString.h"
#include "SFFloat.h"
#include "Scene.h"

static const char	    *fogTypes[] = {"LINEAR", "EXPONENTIAL", NULL};

ProtoFog::ProtoFog(Scene *scene)
  : Proto(scene, "Fog")
{
    addExposedField(SFCOLOR, "color", new SFColor(1.0f, 1.0f, 1.0f));
    addExposedField(SFSTRING, "fogType", new SFString("LINEAR"), FF_FIXEDSTRINGS, fogTypes);
    addExposedField(SFFLOAT, "visibilityRange", new SFFloat(0.0f), new SFFloat(0.0f));
    addEventIn(SFBOOL, "set_bind");
    addEventOut(SFBOOL, "isBound");
}

NodeFog::NodeFog(Scene *scene, Proto *def)
  : Node(scene, def)
{
}

void
NodeFog::preDraw()
{
    glPushMatrix();
    glGetFloatv(GL_MODELVIEW_MATRIX, _matrix);
    glPopMatrix();
    _scene->addFog(this);
}

void
NodeFog::apply()
{
    const float	   *color = ((SFColor *) getField(0))->getValue();
    const char	   *type = ((SFString *) getField(1))->getValue();
    float	    visibilityRange = ((SFFloat *) getField(2))->getValue();

    if (visibilityRange > 0.0f) {
	if (!strcmp(type, "LINEAR")) {
	    glFogi(GL_FOG_MODE, GL_LINEAR);
	    glFogf(GL_FOG_END, visibilityRange);
	} else if (!strcmp(type, "EXPONENTIAL")) {
	    glFogi(GL_FOG_MODE, GL_EXP);
	    glFogf(GL_FOG_DENSITY, 2.0f / visibilityRange);
	}
	glFogfv(GL_FOG_COLOR, color);
	glEnable(GL_FOG);
    } else {
	glDisable(GL_FOG);
    }
}
