/*
 * NodeIndexedLineSet.cpp
 *
 * Copyright (C) 1999 Stephen F. White
 * 
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program (see the file "COPYING" for details); if 
 * not, write to the Free Software Foundation, Inc., 675 Mass Ave, 
 * Cambridge, MA 02139, USA.
 */

#include <stdio.h>
#include <GL/gl.h>
#include "stdafx.h"

#include "NodeIndexedLineSet.h"
#include "Proto.h"
#include "FieldValue.h"
#include "SFNode.h"
#include "MFInt32.h"
#include "SFInt32.h"
#include "SFBool.h"
#include "MFColor.h"
#include "MFVec3f.h"

ProtoIndexedLineSet::ProtoIndexedLineSet(Scene *scene)
  : Proto(scene, "IndexedLineSet")
{
    addEventIn(MFINT32, "set_colorIndex");
    addEventIn(MFINT32, "set_coordIndex");
    addExposedField(SFNODE, "color", new SFNode(NULL), NODE_COLOR);
    addExposedField(SFNODE, "coord", new SFNode(NULL), NODE_COORDINATE);
    addField(MFINT32, "colorIndex", new MFInt32(), new SFInt32(-1));
    addField(SFBOOL, "colorPerVertex", new SFBool(true));
    addField(MFINT32, "coordIndex", new MFInt32(), new SFInt32(-1));
}

NodeIndexedLineSet::NodeIndexedLineSet(Scene *scene, Proto *def)
  : Node(scene, def)
{
}

void
NodeIndexedLineSet::draw()
{
    Node       *color = ((SFNode *) getField(0))->getValue();
    Node       *coord = ((SFNode *) getField(1))->getValue();
    MFInt32    *colorIndex = (MFInt32 *) getField(2);
//    int		numColorIndex = colorIndex->getSize();
    bool	colorPerVertex = ((SFBool *) getField(3))->getValue();
    MFInt32    *coordIndex = (MFInt32 *) getField(4);
    MFColor    *colors;
    int		colorSize;

    glPushAttrib(GL_ENABLE_BIT);
    glDisable(GL_LIGHTING);
    glDisable(GL_TEXTURE_2D);
    glLineWidth(1.0f);
    if (color) {
	colors = (MFColor *) color->getField(0);
	colorSize = colors->getSize();
    } else {
	colors = NULL;
	colorSize = 0;
	float	    c[4];
	glGetMaterialfv(GL_FRONT, GL_EMISSION, c);
	glColor3fv(c);
    }

    if (!coord) return;

    MFVec3f    *coords = (MFVec3f *) coord->getField(0);
    int		coordSize = coords->getSize();

    bool	inSet = false;

    int size = coordIndex->getSize();
    for (int i = 0; i < size; i++) {
	int index = coordIndex->getValue(i);
	if (index == -1) {
	    if (inSet) {
		glEnd();
		inSet = false;
	    }
	} else {
	    if (!inSet) {
		glBegin(GL_LINES);
		inSet = true;
	    }
	    if (color) {
		int cindex = colorIndex->getValue(i);
		if (colorPerVertex && cindex < colorSize) {
		    glColor3fv(colors->getValue(cindex));
		}
	    }
	    if (index < coordSize) {
		glVertex3fv(coords->getValue(index));
	    }
	}
    }
    if (inSet) glEnd();
    glPopAttrib();
}
