/*
 * NodeMaterial.cpp
 *
 * Copyright (C) 1999 Stephen F. White
 * 
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program (see the file "COPYING" for details); if 
 * not, write to the Free Software Foundation, Inc., 675 Mass Ave, 
 * Cambridge, MA 02139, USA.
 */

#include <stdio.h>
#include <GL/gl.h>
#include "stdafx.h"

#include "NodeMaterial.h"
#include "Proto.h"
#include "FieldValue.h"
#include "ExposedField.h"
#include "SFFloat.h"
#include "SFColor.h"
#include "Node.h"



ProtoMaterial::ProtoMaterial(Scene *scene)
  : Proto(scene, "Material")
{
    addExposedField(SFFLOAT, "ambientIntensity", new SFFloat(0.2f), 
    		    new SFFloat(0.0f), new SFFloat(1.0f));
    addExposedField(SFCOLOR, "diffuseColor", new SFColor(0.8f, 0.8f, 0.8f));
    addExposedField(SFCOLOR, "emissiveColor", new SFColor(0.0f, 0.0f, 0.0f));
    addExposedField(SFFLOAT, "shininess", new SFFloat(0.2f),
		    new SFFloat(0.0f), new SFFloat(1.0f));
    addExposedField(SFCOLOR, "specularColor", new SFColor(0.0f, 0.0f, 0.0f));
    addExposedField(SFFLOAT, "transparency", new SFFloat(0.0f), 
    		    new SFFloat(0.0f), new SFFloat(1.0f));
}

NodeMaterial::NodeMaterial(Scene *scene, Proto *def)
  : Node(scene, def)
{
}


void NodeMaterial::bind()
{
    const float ambientIntensity = ((SFFloat *) getField(0))->getValue();
    const float *diffuseColor = ((SFColor *) getField(1))->getValue();
    const float *emissiveColor = ((SFColor *) getField(2))->getValue();
    float shininess = ((SFFloat *) getField(3))->getValue();
    const float *specularColor = ((SFColor *) getField(4))->getValue();
    float transparency = ((SFFloat *) getField(5))->getValue();
    float ambientColor[4], dc[4];

    for (int i = 0; i < 3; i++) {
	dc[i] = diffuseColor[i];
	ambientColor[i] = diffuseColor[i] * ambientIntensity;
    }
    ambientColor[3] = 1.0f;
    dc[3] = 1.0f - transparency;

    glMaterialfv(GL_FRONT_AND_BACK, GL_AMBIENT, ambientColor);
    glMaterialfv(GL_FRONT_AND_BACK, GL_DIFFUSE, dc);
    glMaterialfv(GL_FRONT_AND_BACK, GL_EMISSION, emissiveColor);
    glMaterialf(GL_FRONT_AND_BACK, GL_SHININESS, shininess * 128.0f);
    glMaterialfv(GL_FRONT_AND_BACK, GL_SPECULAR, specularColor);

    if (transparency > 0.0f) {
	glBlendFunc(GL_SRC_ALPHA, GL_ONE_MINUS_SRC_ALPHA);
	glEnable(GL_BLEND);
    }
}

void NodeMaterial::unbind()
{
    const float ambientIntensity = ((SFFloat *) getProto()->getExposedField(0)->getValue())->getValue();
    const float *diffuseColor = ((SFColor *) getProto()->getExposedField(1)->getValue())->getValue();
    const float *emissiveColor = ((SFColor *) getProto()->getExposedField(2)->getValue())->getValue();
    const float shininess = ((SFFloat *) getProto()->getExposedField(3)->getValue())->getValue();
    const float *specularColor = ((SFColor *) getProto()->getExposedField(4)->getValue())->getValue();
    float ambientColor[4];

    for (int i = 0; i < 3; i++) {
	ambientColor[i] = diffuseColor[i] * ambientIntensity;
    }
    ambientColor[3] = 1.0f;

    glMaterialfv(GL_FRONT_AND_BACK, GL_AMBIENT, ambientColor);
    glMaterialfv(GL_FRONT_AND_BACK, GL_DIFFUSE, diffuseColor);
    glMaterialfv(GL_FRONT_AND_BACK, GL_EMISSION, emissiveColor);
    glMaterialf(GL_FRONT_AND_BACK, GL_SHININESS, shininess);
    glMaterialfv(GL_FRONT_AND_BACK, GL_SPECULAR, specularColor);

    glDisable(GL_BLEND);
}
