/*
 * NodeNurbsSurface.h
 *
 * Copyright (C) 1999 Stephen F. White
 * 
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program (see the file "COPYING" for details); if 
 * not, write to the Free Software Foundation, Inc., 675 Mass Ave, 
 * Cambridge, MA 02139, USA.
 */

#ifndef _NODE_NURBS_SURFACE_H
#define _NODE_NURBS_SURFACE_H

#ifndef _NODE_H
#include "Node.h"
#endif
#ifndef _PROTO_H
#include "Proto.h"
#endif

class Vec3f;
class Mesh;

class NodeNurbsSurface : public Node {
public:
		    NodeNurbsSurface(Scene *scene, Proto *proto);
protected:
    virtual	   ~NodeNurbsSurface();

public:
    virtual int	    getType() const { return NODE_NURBS_SURFACE; }
    virtual Node   *copy() const { return new NodeNurbsSurface(*this); }

    virtual void    draw();
    virtual void    drawHandles();

    virtual Vec3f   getHandle(int handle, int *constraint, int *field);
    virtual void    setHandle(int handle, const Vec3f &v);

    virtual void    setField(int index, FieldValue *value);

protected:
    void	    createMesh();
    static int	    findSpan(int dimension, int order, float u,
    			     const float knots[]);
    static void     basisFuns(int span, float u, int order,
    			      const float knots[], float basis[],
			      float deriv[]);
    static Vec3f    surfacePoint(int uDimension, int uOrder,
    				 const float uKnots[],
                                 int vDimension, int vOrder,
				 const float vKnots[],
			         const Vec3f controlPoints[],
				 const float weights[],
				 float u, float v, Vec3f &normal);
private:
    Mesh	   *_mesh;
    bool	    _meshDirty;
};

class ProtoNurbsSurface : public Proto {
public:
		    ProtoNurbsSurface(Scene *scene);
    virtual Node   *create(Scene *scene)
		        { return new NodeNurbsSurface(scene, this); }
};

#endif // _NODE_NURBS_SURFACE_H
