/*
 * NodeScript.cpp
 *
 * Copyright (C) 1999 Stephen F. White
 * 
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program (see the file "COPYING" for details); if 
 * not, write to the Free Software Foundation, Inc., 675 Mass Ave, 
 * Cambridge, MA 02139, USA.
 */

#include <stdio.h>
#include "stdafx.h"

#include "NodeScript.h"
#include "Scene.h"
#include "Proto.h"
#include "FieldValue.h"
#include "MFString.h"
#include "SFBool.h"
#include "ExposedField.h"
#include "Field.h"


ProtoScript::ProtoScript(Scene *scene)
  : Proto(scene, "Script")
{
    addExposedField(MFSTRING, "url", new MFString(), FF_HIDDEN | FF_URL, NULL);
    addField(SFBOOL, "directOutput", new SFBool(false), FF_HIDDEN, NULL);
    addField(SFBOOL, "mustEvaluate", new SFBool(false), FF_HIDDEN, NULL);
}

NodeScript::NodeScript(Scene *scene)
  : Node(scene, new ProtoScript(scene))
{
}

NodeScript::~NodeScript()
{
    delete _proto;
}

void
NodeScript::write(FILE *f, int indent)
{
    indentf(f, indent);
    if (needsDEF()) {
	if (!_name[0]) _scene->generateUniqueNodeName(this);
	fprintf(f, "DEF %s ", (const char *) _name);
    }
    fprintf(f, "Script {\n");
    writeFields(f, indent + 2);
    _proto->write(f, indent + 2);
    indentf(f, indent);
    writeRoutes(f, indent+2);
    indentf(f, indent);
    fprintf(f, "}\n");
}

// since scripts can have dynamically-added fields, we need to update
// the node's fields after the script is defined

void
NodeScript::update()
{
    int	    i, newNumFields = _proto->getNumFields();

    if (_numFields != newNumFields) {
	FieldValue	**newFields = new FieldValue *[newNumFields];
	for (i = 0; i < _numFields; i++) {
	    newFields[i] = _fields[i];
	}
	for (i = _numFields; i < newNumFields; i++) {
	    newFields[i] = _proto->getField(i)->getDefault();
	    newFields[i]->ref();
	}
	delete [] _fields;
	_fields = newFields;
	_numFields = newNumFields;
    }
    if (_numEventIns != _proto->getNumEventIns()) {
	delete [] _inputs;
	_inputs = new SocketList [_proto->getNumEventIns()];
	_numEventIns = _proto->getNumEventIns();
    }
    if (_numEventOuts != _proto->getNumEventOuts()) {
	delete [] _outputs;
	_outputs = new SocketList [_proto->getNumEventOuts()];
	_numEventOuts = _proto->getNumEventOuts();
    }
}
