/*
 * Proto.h
 *
 * Copyright (C) 1999 Stephen F. White
 * 
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program (see the file "COPYING" for details); if 
 * not, write to the Free Software Foundation, Inc., 675 Mass Ave, 
 * Cambridge, MA 02139, USA.
 */

#ifndef _PROTO_H
#define _PROTO_H

#ifndef _ARRAY_H
#include "Array.h"
#endif
#ifndef _DUNE_STRING_H
#include "String.h"
#endif
#ifndef _NODE_H
#include "Node.h"
#endif

#define INVALID_INDEX (-1)

class Element;
class EventIn;
class EventOut;
class Field;
class ExposedField;
class FieldValue;
class Scene;
class Node;
class NodeList;
typedef Array<String> StringArray;

class Proto {
public:
				Proto(Scene *scene, const String &name);
    virtual		       ~Proto();

    virtual Node	       *create(Scene *scene);

    EventIn		       *getEventIn(int index) const { return _eventIns[index]; }
    EventOut		       *getEventOut(int index) const { return _eventOuts[index]; }
    Field		       *getField(int index) const { return _fields[index]; }
    ExposedField	       *getExposedField(int index) const { return _exposedFields[index]; }

    int				getNumFields() const { return _fields.size(); }
    int				getNumEventIns() const { return _eventIns.size(); }
    int				getNumEventOuts() const { return _eventOuts.size(); }

    int				lookupEventIn(const String &name) const;
    int				lookupEventOut(const String &name) const;
    int				lookupField(const String &name) const;
    int				lookupExposedField(const String &name) const;

    const String	       &getName() const { return _name; }

    void			write(FILE *f, int indent) const;

    void			addElement(Element *element);
    void			addExposedField(ExposedField *exposedField);

    void			define(Node *primaryNode, NodeList *nodes);

protected:
    void			addField(int fieldType, const String &name,
    					 FieldValue *defaultValue,
					 FieldValue *min = NULL,
					 FieldValue *max = NULL);
    void			addField(int fieldType, const String &name,
    					 FieldValue *defaultValue,
					 int nodeType);
    void			addField(int fieldType, const String &name,
					 FieldValue *defaultValue,
					 int flags,
					 const char **strings);
    void			addEventIn(int fieldType, const String &name);
    void			addEventOut(int fieldType, const String &name);
    void			addExposedField(int fieldType,
						const String &name,
    						FieldValue *defaultValue,
						FieldValue *min = NULL,
						FieldValue *max = NULL);
    void			addExposedField(int fieldType,
						const String &name,
    						FieldValue *defaultValue,
						int nodeType);
    void			addExposedField(int fieldType,
						const String &name,
    						FieldValue *defaultValue,
						int flags,
						const char **strings);
    int				lookupSimpleEventIn(const String &name) const;
    int				lookupSimpleEventOut(const String &name) const;

protected:
    Scene		       *_scene;
    String			_name;
    Array<EventIn *>		_eventIns;
    Array<EventOut *>		_eventOuts;
    Array<Field *>		_fields;
    Array<ExposedField *>	_exposedFields;

    // for PROTO's:
    NodeList		       *_nodes;
    Node		       *_primaryNode;

};

class ProtoNode : public Node
{
public:
			ProtoNode(Scene *scene, Proto *proto);

    virtual int		getType() const { return -1; }
    virtual Node       *copy() const { return new ProtoNode(*this); }
};

#endif // _PROTO_H
