/*
 * SFColor.cpp
 *
 * Copyright (C) 1999 Stephen F. White
 * 
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program (see the file "COPYING" for details); if 
 * not, write to the Free Software Foundation, Inc., 675 Mass Ave, 
 * Cambridge, MA 02139, USA.
 */

#include <stdio.h>
#include "stdafx.h"

#include "SFColor.h"
#include "SFFloat.h"

SFColor::SFColor(float r, float g, float b)
{
    _value[0] = r; _value[1] = g; _value[2] = b;  _value[3] = 1.0f;
}

SFColor::SFColor(const float *values)
{
    _value[0] = values[0];
    _value[1] = values[1];
    _value[2] = values[2];
    _value[3] = 1.0f;
}

bool
SFColor::equals(const FieldValue *value) const
{
    if (value->getType() == SFCOLOR) {
        for (int i = 0; i < 3; i++)
            if (((SFColor *) value)->getValue()[i] != _value[i])
                return false;
        return true;
    }
    return false;
}

void SFColor::write(FILE *f, int /* indent */) const
{
    fprintf(f, "%g %g %g\n", _value[0], _value[1], _value[2]);
}

void
SFColor::clamp(const FieldValue *min, const FieldValue *max)
{
    for (int i = 0; i < 3; i++) {
	if (_value[i] < 0.0f) _value[i] = 0.0f;
	if (_value[i] > 1.0f) _value[i] = 1.0f;
    }
}
