/*
 * String.h
 *
 * Copyright (C) 1999 Stephen F. White
 * 
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program (see the file "COPYING" for details); if 
 * not, write to the Free Software Foundation, Inc., 675 Mass Ave, 
 * Cambridge, MA 02139, USA.
 */

#ifndef _DUNE_STRING_H
#define _DUNE_STRING_H

#ifndef _STRING_H
#include <string.h>
#endif
#ifndef _MALLOC_H
#include <malloc.h>
#endif

class String {

    class StringBuf {
    public:
	friend class String;
		StringBuf(const char *d)
		{ data = strdup(d); refs = 1; len = strlen(d); capacity = len + 1; }
	       ~StringBuf()
		{ free(data); }
    protected:
	char   *data;
	int	refs;
	int	len;
	int	capacity;
    };

    public:
		String();
		String(const char *str);
		String(char c);
		String(const String &s);
	       ~String();

    String     &operator =(const String &s);
    String     &operator +=(char c);
    String     &operator +=(const char *s);
	        operator const char *() const { return _stringBuf->data; }
    int		operator ==(const String &str) const;
    int		length() const { return _stringBuf->len; }

protected:
    StringBuf  *_stringBuf;
};

extern int hash(String key);

#endif // _DUNE_STRING_H
