/*
 * lexer.l
 *
 * Copyright (C) 1999 Stephen F. White
 * 
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program (see the file "COPYING" for details); if 
 * not, write to the Free Software Foundation, Inc., 675 Mass Ave, 
 * Cambridge, MA 02139, USA.
 */

%{
#include <stdlib.h>
#include <string.h>
#include "stdafx.h"

#include "Element.h"
#include "EventIn.h"
#include "EventOut.h"
#include "ExposedField.h"
#include "Field.h"
#include "FieldValue.h"
#include "Node.h"
#include "Scene.h"

#include "SFBool.h"
#include "SFNode.h"
#include "MFNode.h"
#include "SFString.h"

#include "parser.h"
#include "y.tab.h"

#define YY_NEVER_INTERACTIVE 1
#define YY_INPUT(buf, result, max_size) \
	{ result = fread( buf, 1, max_size, inputFile ); }

#if defined(__sgi) && !defined(__GNUC__)
#pragma set woff 1110
#endif

%}

IDFIRSTCHAR	[^ \r\n\t'\"#,\.\[\]\\\{\}0-9\+\-]
IDRESTCHARS	[^ \r\n\t'\"#,\.\[\]\\\{\}]

EXPONENT	([Ee]("-"|"+")?[0-9]+)

%%

("-"|"+")?([0-9]+"."[0-9]*|[0-9]*"."[0-9]+){EXPONENT}? { yylval.sffloat = (float) atof(yytext); return FLOAT; }

("-"|"+")?[0-9]+{EXPONENT}	{ yylval.sffloat = (float) atof(yytext); return FLOAT; }

("-"|"+")?[0-9]+		{ yylval.int32 = atoi(yytext); return INT; }

("-"|"+")?0x[0-9a-fA-F]+	{ yylval.int32 = strtol(yytext, NULL, 16); return INT; }

Script		{ return SCRIPT; }

DEF		{ return DEF; }

EXTERNPROTO	{ return EXTERNPROTO; }

FALSE		{ return FALSE_TOK; }

IS		{ return IS; }

NULL		{ return NULL_TOK; }

PROTO		{ return PROTO; }

ROUTE		{ return ROUTE; }

TO		{ return TO; }

TRUE		{ return TRUE_TOK; }

USE		{ return USE; }

eventIn		{ return EVENT_IN; }

eventOut	{ return EVENT_OUT; }

exposedField	{ return EXPOSED_FIELD; }

field		{ return FIELD; }

{IDFIRSTCHAR}{IDRESTCHARS}*	{
		  yylval.id = scene->addSymbol(yytext); return ID;
		}

\"([^\"]|\\\"|\\\\)*\"	{
		  for (const char *s = yytext; *s; s++) {
		      if (*s == '\n') lineno++;
		  }
		  yytext[strlen(yytext)-1] = '\0';
		  yylval.id = scene->addSymbol(yytext+1);
		  return STRING;
		}

"#"[^\n]*	/* eat up comments */

[ \t\r,]+	/* eat up whitespace and commas */

\n		{ lineno++; }

.		{ return yytext[0]; }

%%
